var noble = require('noble');

var exitHandlerBound = false;
var maxPeripherals = 1;

var peripherals = [];

var http = require('http');
var server = http.createServer(function(req, res){});

server.listen(1234, function(){
    console.log((new Date()) + " Server is listening on port 1234");
});

var WebSocketServer = require('websocket').server;
var utf8 = require('utf8');

//passing the server we created in http to the websocket server
wsServer = new WebSocketServer({
    httpServer: server
});

var client;


 /////////////websocket part////////////////
//////////////sends to bean ///////////////

wsServer.on('request', function(request){
   //run on connection and store it
    client = request.accept(null, request.origin);
    console.log((new Date()) + " Connection Accepted");
    
    
    //user disconnects (ws user)
    client.on('close', function(reasonCode, description){
       delete client;
        console.log((new Date()) + " " + client.remoteAddress + " disconnected.");
    });
});//on request


noble.on('stateChange', function(state){
   //possible states: unknown, resetting, unsupported, unauthorized, poweredOff, poweredOn
    if(state === 'poweredOn'){
        console.log('Powered On!');
        //for next line, 
        //  false = do not allow multiple devices diferentiated by UUID
        //limit to devices having the service UUID request below which all bean's have
        //scanning for beans only!
        noble.startScanning(['a495ff10c5b14b44b5121370f02d74de'], false);
      
    }
});

//if bean is discovered
noble.on('discover', function(peripheral){
    //check the name to see if its the one we want
    //could add more devices ["name1","name2", "name3"]
    var desiredDevices = ["🤖's Bean+"];
    //if it it's the beans we are looking for
    if (peripheral.advertisement.localName && desiredDevices.indexOf(peripheral.advertisement.localName) > -1){
        //print some info
        console.log("Found Device with local name: " + peripheral.advertisement.localName);
        console.log("Device UUID: " + peripheral.uuid);
        console.log("Advertising Services: " + peripheral.advertisement.serviceUuids);
        console.log();
        //connect to the bean
        peripheral.connect(connect.bind({peripheral: peripheral}));
    }
    
});

var connect = function(err){
    if (err) throw err;
    console.log("Connection to "+this.peripheral.uuid);
    
    //can be changed if you are using multiple peripherals
    peripherals[0] = {};
    peripherals[0]['peripheral'] = this.peripheral;
    
    //stop discovering
    if(peripherals.length >= maxPeripherals){
        noble.stopScanning();
        console.log("Stopping BLE scan. Reached " + maxPeripherals + " peripherals.");
    }
    if(!exitHandlerBound){
        exitHandlerBound = true;
        process.on('SIGINT', exitHandler);
    }
    this.peripheral.discoverServices([], setupService);
};

//setup notification which begins with discovering the services for the device
//find the right one, then find the characteristic we want
var setupService = function(err, services){
    if(err) throw err;
    //for each of the services
    services.forEach(function(service){
        if (service.uuid === 'a495ff20c5b14b44b5121370f02d74de'){
            console.log("Found Bean Scratch UUID");
            //'a495ff2----skecth#-----c5b14b44b5121370f02d74de'
            var characteristicUUIDS = ['a495ff21c5b14b44b5121370f02d74de',
                                       'a495ff22c5b14b44b5121370f02d74de',
                                       'a495ff23c5b14b44b5121370f02d74de',
                                       'a495ff24c5b14b44b5121370f02d74de',
                                       'a495ff25c5b14b44b5121370f02d74de'
                                      ];
            
            service.discoverCharacteristics(characteristicUUIDS, function(err, characteristics){
                console.log("Got Characteristics.");
                peripherals[0]['characteristics'] = characteristics;
                requestNotify(characteristics[1]);//second scratch characteristics (recieving)
                
                //requestNotify(characteristics[0]);//this first sketch characteristic (runs 0-3 instead of 1-4)
            });//discoverCharacteristics
        }
    });//forEach
};//setupService

var requestNotify = function(characteristic){
    //bind a read handler for updates - so when data arrives we get a callback
    characteristic.on('read', function(data, isNotification){
        var dataString = data.toString("ascii").trim();
        console.log("Data recieved: " + dataString);
        
        if (client){
            client.sendUTF(utf8.encode(dataString));
        }
       
        
    });//read
    //turn on notifications
    characteristic.notify(true, function(err){
        console.log("Turned on notifications "+(err ? "with error" : "without error"));
        if (client){
            client.sendUTF(utf8.encode("Connected"));
        }
    });
};//request notify

var exitHandler = function(){
    peripherals.forEach(function(peripheral){
        console.log("Disconnecting from " + peripheral['peripheral'].uuid + "....");
        peripheral['peripheral'].disconnect(function(){
            console.log("disconnected");
        });
    });
    
    //end process after 2 more secs
    setTimeout(function(){process.exit();}, 2000);
};

process.stdin.resume(); //so the program wont close instantly